/*
	$Id: menu_applet.c,v 1.43 2004/04/25 20:35:36 Treize Exp $
   ROX-Menu
	Simple application menu applet for ROX desktop environment.

   Copyright (C) 2001 Leandro Pereira <leandro@linuxmag.com.br>
	Copyright (C) 2002, 2003 Peter Geer <geerp@cs.sunyit.edu>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	oundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <unistd.h>
#include <malloc.h>
#include <pwd.h>
#include <signal.h>
#include <ctype.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif
#ifdef HAVE_LIBINTL_H
#include <libintl.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

/* Includes from ROX-CLib */
#include <rox/options.h>
#include <rox/choices.h>
#include <rox/infowin.h>
#include <rox/error.h>
#include <rox/rox_path.h>
#include <rox/rox_filer_action.h>
#include <rox/rox_dnd.h>

#include "menu_images.h"
#include "roxmenu.h"

#include "Icons/menu.xpm"
#include "Icons/menu2.xpm"

/* Leaf names in choices directory. */
#define MENU_ROOT_LEAF  "Menu_Root"
#define MENU_RUN_LEAF   "MenuRun"
#define PID_FILE        "pid.tmp"
#define PID_ENV         "PID_FILE_PATH"

#define SCALED_IMAGE_WIDTH  28
#define SCALED_IMAGE_HEIGHT 28

#define DEFAULT_ICON_WIDTH  60
#define DEFAULT_ICON_HEIGHT 38

#define ENTER 0
#define LEAVE 1

#define TOPLEVEL_NO      0
#define TOPLEVEL_YES     1
#define TOPLEVEL_NO_FILE 2

/* Function Prototypes */
static int build_menu(void);
static void opts_changed(void);
static void change_menu_pixmap(gint state);
static void app_exit(void);
static void sys_menu_popup(GdkEventButton *bev, gpointer win);
static gboolean update_menu_timer(RoxMenuDir *root);

/* Module data */
static RoxMenuDir *menu_structure;
static GtkWidget *window;
guchar *app_name;            /* Name of AppDir (from environment) */
static GtkWidget *button, *_pixmap; /* Menu button and its icon. */
static gulong update_timeout_id = 0;

/* Global data */
/* These variables are used as extern by the other modules. */

/* Default pixbufs */
GdkPixbuf *default_group_pixmap=NULL;
GdkPixbuf *default_entry_pixmap=NULL;
GdkPixbuf *default_file_pixmap=NULL;

/* Options declarations. */
Option icon_height;
Option icon_width;
Option menu_file_on;
Option menu_file_off;
Option groups_on_bottom;
Option strip_number;
Option show_files;
Option group_image;
Option entry_image;
Option file_image;
Option entry_width;
Option entry_height;
Option scale_up;
Option show_directions;
Option update_frequency;

static void setup_options(void) {
	option_add_int(&icon_height, "icon_height", DEFAULT_ICON_HEIGHT);
	option_add_int(&icon_width, "icon_width", DEFAULT_ICON_WIDTH);
	option_add_string(&menu_file_on, "menu_file_on", "");
	option_add_string(&menu_file_off, "menu_file_off", "");
	option_add_int(&groups_on_bottom, "groups_on_bottom", 0);
	option_add_int(&strip_number, "strip_number", 0);
	option_add_int(&show_files, "show_files", 0);
	option_add_string(&group_image, "group_image", "");
	option_add_string(&entry_image, "entry_image", "");
	option_add_string(&file_image, "file_image", "");
	option_add_int(&entry_height, "entry_height", SCALED_IMAGE_HEIGHT);
	option_add_int(&entry_width, "entry_width", SCALED_IMAGE_WIDTH);
	option_add_int(&scale_up, "scale_up", 1);
	option_add_int(&show_directions, "show_directions_dialog", 1);
	option_add_int(&update_frequency, "update_frequency", 5);
	option_add_notify(opts_changed);
}

static void opts_changed(void) {
	if (!menu_structure || menu_structure->is_updating) {
		return;
	}
	if (menu_file_on.has_changed || menu_file_off.has_changed ||
	    icon_height.has_changed || icon_width.has_changed) {
		change_menu_pixmap(LEAVE);
	}
	if (group_image.has_changed || entry_image.has_changed || 
	    file_image.has_changed ||
	    entry_width.has_changed || entry_height.has_changed) {
		if (GDK_IS_PIXBUF(default_group_pixmap) ) gdk_pixbuf_unref(default_group_pixmap);
		if (GDK_IS_PIXBUF(default_entry_pixmap) ) gdk_pixbuf_unref(default_entry_pixmap);
		if (GDK_IS_PIXBUF(default_file_pixmap) ) gdk_pixbuf_unref(default_file_pixmap);
		default_group_pixmap = NULL;
		default_entry_pixmap = NULL;
		default_file_pixmap = NULL;
	}
	if (strip_number.has_changed || groups_on_bottom.has_changed) {
		menudir_set_grouping(menu_structure, 
	                     groups_on_bottom.int_value ? 
	                     GROUPS_ON_BOTTOM : GROUPS_ON_TOP, TRUE);
		build_menu();
	}
	if (show_files.has_changed) {
		menudir_set_show_files(menu_structure, 
		                       show_files.int_value?TRUE:FALSE, TRUE);
		menudir_update(menu_structure);
	}
	if (update_frequency.has_changed) {
		if (update_timeout_id > 0) gtk_timeout_remove(update_timeout_id);
		update_timeout_id = gtk_timeout_add(update_frequency.int_value*1000, 
		                (GtkFunction)update_menu_timer, menu_structure);
	}
}

static gboolean update_menu_timer(RoxMenuDir *root) {
	menudir_update(root);
	return TRUE;
}

static int build_menu(void) {

	gchar *user_choices, *menu_root;
	gchar *path;
	int status;

	if (update_timeout_id > 0) gtk_timeout_remove(update_timeout_id);
	
	/* Clear the MIME image list. */
	clear_image_list();
	/* Destroy the old menu, if it exists. */
	if (menu_structure) menudir_destroy(menu_structure);

	/* Get the menu directory and run the MenuRun script. */
	user_choices = (gchar*)choices_find_path_load(NULL, 
	                                       (const char*)app_name);
	menu_root = g_build_path(G_DIR_SEPARATOR_S, user_choices, MENU_ROOT_LEAF, NULL);
	path = g_build_path(G_DIR_SEPARATOR_S, user_choices, MENU_RUN_LEAF, NULL);
	g_free(user_choices);
	
	if (g_file_test(path, G_FILE_TEST_IS_EXECUTABLE)) {
		/* We use system because we don't want to continue execution
		   until the MenuRun script has finished. */
		status = system(path);
		if (status != 0) fprintf(stderr, 
		                    ("MenuRun exited with status %d\n"), status);
	}
	g_free(path);

	menu_structure = menudir_toplevel_new(menu_root);
	g_free(menu_root);
	menudir_set_grouping(menu_structure, 
	                     groups_on_bottom.int_value ? 
	                     GROUPS_ON_BOTTOM : GROUPS_ON_TOP, TRUE);
	menudir_set_show_files(menu_structure, 
	                       show_files.int_value?TRUE:FALSE, TRUE);
	                     
	menudir_update(menu_structure);
	update_timeout_id = gtk_timeout_add(update_frequency.int_value*1000, 
	                (GtkFunction)update_menu_timer, menu_structure);
		
	if (menudir_is_empty(menu_structure)) {
		rox_error(_("Your menu is empty.  Add items to it and refresh."));
	}
	
   return 1;
}

static void change_menu_pixmap(gint state) {
	gchar *ico;
	char ** def;

	g_return_if_fail(window != NULL);

	if (GPOINTER_TO_INT(state) == ENTER) {
		ico = menu_file_on.value;
		def = menu2_xpm;
	} else {
		ico = menu_file_off.value;
		def = menu_xpm;
	}
	
   if (GTK_IS_WIDGET(_pixmap) ) gtk_widget_destroy(_pixmap);
		/*gtk_container_remove(GTK_CONTAINER(button), _pixmap);*/
   _pixmap = create_pixmap(ico, def, icon_width.int_value, 
	                                  icon_height.int_value, TRUE);
   gtk_widget_show(_pixmap);
   gtk_container_add(GTK_CONTAINER(button), _pixmap);
}

static void set_position(GtkMenu * _menu, gint * px, gint * py, gpointer data)
{
	gint scr_width, scr_height;
	GtkRequisition requisition, btn_req;
	GdkDisplay *disp;  /* I don't think I'm supposed to free this...*/
	GdkModifierType modmask;

	scr_width = gdk_screen_width();
	scr_height = gdk_screen_height();

	gtk_widget_size_request(GTK_WIDGET(_menu), &requisition);

	if (!window) {
		
		disp = gdk_display_get_default();
		gdk_display_get_pointer(disp, NULL, px, py, &modmask);

		if ( (*py + requisition.height) > scr_height) {
			*py = scr_height - requisition.height;
			*py = *py < 0 ? 0 : *py;
		}
		if ( (*px + requisition.width) > scr_width) {
			*px = scr_width - requisition.width;
			*px = *px < 0 ? 0 : *px;
		}
		
	} else {
	
		gtk_widget_size_request(GTK_WIDGET(button), &btn_req);
		
		gdk_window_get_origin(button->window, px, py);

		if (*py > requisition.height) *py -= requisition.height;
		else *py += btn_req.height;

		if (btn_req.height + requisition.height > scr_height)
			*py -= requisition.height;

		if (*px + requisition.width > scr_width)
			*px = *px - (requisition.width - btn_req.width);
	}
}

static void default_choices_setup(void) {
	
	guchar *choicesdir;
	guchar *menu_dir_path, *dest_dir, *home_apps;
	int ret, status = 1;
	
	choicesdir = (gchar*)choices_find_path_save(NULL, app_name,FALSE);
	if (choicesdir == NULL) 
		choicesdir = (gchar*)choices_find_path_load(NULL, app_name);
	menu_dir_path = g_build_path(G_DIR_SEPARATOR_S, choicesdir, MENU_ROOT_LEAF, NULL);	
	
	/* Exit if we already have a choices directory. */
	if (g_file_test(menu_dir_path, G_FILE_TEST_IS_DIR)) return;
	
	/* The directory does not exist, so try to create it with some
	   reasonable default links. */
	ret = mkdir(choicesdir, 0777);
	if (ret == 0) ret = mkdir(menu_dir_path, 0777);
	if (ret != 0) {
		rox_error(
			("Fatal error: Unable to create configuration directory %s.\n"),
			menu_dir_path);
		app_exit();
		exit(1);
	}
	dest_dir = g_build_path(G_DIR_SEPARATOR_S, menu_dir_path, ("SystemApps"), NULL);
	ret = symlink("/usr/apps", dest_dir);
	if (ret == 0) status = 0;
	g_free(dest_dir);
	dest_dir = g_build_path(G_DIR_SEPARATOR_S, menu_dir_path, ("LocalApps"), NULL);
	ret = symlink("/usr/local/apps", dest_dir);
	if (ret == 0) status = 0;
	g_free(dest_dir);
	dest_dir = getenv("HOME");
	home_apps = g_build_path(G_DIR_SEPARATOR_S, dest_dir, "Apps", NULL);
	g_free(dest_dir);
	dest_dir = g_build_path(G_DIR_SEPARATOR_S, menu_dir_path, ("UserApps"), NULL);
	ret = symlink(home_apps, dest_dir);
	if (ret == 0) status = 0;
	g_free(dest_dir);
	g_free(home_apps);
	if (status != 0) 
		rox_error(_("Unable to create default menu.  You will need to add \
		applications manually.  Please refer to the README file."));
}

/* This is called as a signal handler */
static void child_died(int signum)
{
	pid_t	child;
	int	status;
	/* Find out which children exited and allow them to die */
	while (1) {
		child = waitpid(-1, &status, WNOHANG);
		if (child == 0 || child == -1) break;
	}
}

/* Handler for SIGUSR1 and SIGUSR2. */
static void menu_popup (int sig) {

	if (sig == SIGUSR1) {
		gtk_menu_popup(GTK_MENU(menu_structure->visible_menu), NULL, NULL,
		               (void*)set_position, NULL, 1, 
		               gtk_get_current_event_time());
		if (window) gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
	} else if (sig == SIGUSR2) {
		sys_menu_popup(NULL, NULL);
	} else {
		app_exit();
		exit(0);
	}
}

/* 
	Some signal handlers for destroying the window.  I found it annoying
	that clicking the WM's 'X' left the program running.	-P.G.
*/
static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
	return(FALSE);
}

static void quit_app(void) {
	gtk_main_quit();
}

static void destroy(GtkWidget *widget, gpointer data) { 
	quit_app(); 
}

static void refresh_menu(void) {
	build_menu();	
}

static void show_options_win(void) {
	options_show();
}

static void show_choices(void) {

	gchar *choices_path;
	gchar *dir;
	static GtkWidget *directions_dialog;
	
	choices_path= (gchar*)choices_find_path_load(NULL, (const char*)app_name);
	dir = g_strconcat(choices_path, "Menu_Root", NULL);

	if (show_directions.int_value) {
		directions_dialog = gtk_message_dialog_new(GTK_WINDOW(window), 
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_INFO,
			GTK_BUTTONS_CLOSE,
			("To add programs to your menu, link AppDirs or folders into \
this folder, which I will show now:\n%s\n\n\
This folder will reflect the contents of your menu.  Each \
application will be displayed in the menu as an application and \
each folder will show as a submenu.  You can create nested \
submenus by putting folders inside of folders.  You can also \
display regular files in the menu by setting an option."), 
			dir);
		gtk_dialog_run(GTK_DIALOG(directions_dialog));
		gtk_widget_destroy(directions_dialog);
	}

	if ( g_file_test(dir, G_FILE_TEST_IS_DIR) ) {
		rox_filer_open_dir((const char*)dir);
	} else {
		rox_error(_("Menu configuration directory does not exist."));
	}
		
	g_free(dir);
	g_free(choices_path);
	
}

static void show_help(void) {
	gchar *app_dir_path;
	app_dir_path = g_build_path(G_DIR_SEPARATOR_S, g_getenv("APP_DIR"), "Help", NULL);
	rox_filer_open_dir((const char*)app_dir_path);
	g_free(app_dir_path);
}

/* Make a destroy-frame into a close, allowing us to re-use the window */
static int trap_frame_destroy(GtkWidget *widget, GdkEvent *event,
			      gpointer data)
{
  /* Change this destroy into a hide */
  gtk_widget_hide(GTK_WIDGET(data));
  return TRUE;
}

/* Show the info window */
static void show_info_win(void)
{
	static GtkWidget *infowin=NULL;
	if(!infowin) {
	infowin=info_win_new(PROJECT, PURPOSE, VERSION, AUTHOR, WEBSITE);
	gtk_signal_connect(GTK_OBJECT(infowin), "delete_event", 
	                   GTK_SIGNAL_FUNC(trap_frame_destroy), infowin);
  }

  gtk_widget_show(infowin);
}

/* Menu stuff borrowed from DemoApplet by Stephen Watson. */
static GtkItemFactoryEntry menu_items[] = {
  { ("/Refresh"), NULL, refresh_menu, 0, "<StockItem>", GTK_STOCK_REFRESH },
  { ("/sep"), NULL, NULL, 0, "<Separator>" },
  { ("/Options..."), NULL, show_options_win, 0, "<StockItem>", GTK_STOCK_PREFERENCES },
  { ("/Menu setup..."), NULL, show_choices, 0, NULL },
  { ("/sep"), NULL, NULL, 0, "<Separator>" },
  { ("/Help"), NULL, show_help, 0, "<StockItem>", GTK_STOCK_HELP },
  { ("/Info"), NULL, show_info_win, 0, "<StockItem>", GTK_STOCK_DIALOG_INFO  },
  { ("/sep"), NULL, NULL, 0, "<Separator>" },
  { ("/Quit"), NULL, quit_app, 0, "<StockItem>", GTK_STOCK_QUIT }
};

static GtkWidget *menu_create_menu(GtkWidget *window)
{
  GtkWidget *context_menu;
  GtkItemFactory	*item_factory;
  GtkAccelGroup	*accel_group;
  gint  n_items = sizeof(menu_items) / sizeof(*menu_items);

  accel_group = gtk_accel_group_new();

  item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<system>", 
				      accel_group);

  gtk_item_factory_create_items(item_factory, n_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  if (window) gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

  context_menu = gtk_item_factory_get_widget(item_factory, "<system>");

  return context_menu;
}

static void sys_menu_popup(GdkEventButton *bev, gpointer win) {
	static GtkWidget *sys_menu;
	if (sys_menu == NULL) sys_menu=menu_create_menu(GTK_WIDGET(win));
	
	if (bev == NULL) {
		gtk_menu_popup(GTK_MENU(sys_menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
	} else {
		gtk_menu_popup(GTK_MENU(sys_menu), NULL, NULL, NULL, NULL, bev->button, bev->time);
	}
}

static gint button_press(GtkWidget *btn, GdkEventButton *bev, gpointer win) {

	if (bev->type == GDK_BUTTON_PRESS && bev->button == 3) {
		sys_menu_popup(bev, win);
		return TRUE;
	
	} else if (bev->type == GDK_BUTTON_PRESS && bev->button == 1) {

		gtk_menu_popup(GTK_MENU(menu_structure->visible_menu), NULL, NULL,
		               (void*)set_position, NULL, bev->button, bev->time);
		gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
		return TRUE;
	
	}
	return FALSE;
}

static rox_dnd_handle_uris dnd_callback(GtkWidget *widget, GSList *uris,
		gpointer data,	gpointer udata) {

	char cmd[]="rox";
	char *argv[256];
	pid_t child;
	GSList *list, *cn;
	int i=1;
	
	child = fork();

	if (child == -1) return FALSE;
	else if (child == 0) {
	
		argv[0]=cmd;
		list = rox_dnd_filter_local(uris);
		cn = list;
	
		while (cn) {
			/*printf("Processing %s\n", cn->data);*/
			argv[i++] = rox_path_get_local((gchar*)cn->data);
			cn=cn->next;
		
		}
		
		argv[i] = NULL;
		rox_dnd_local_free(list);
		
		execvp(cmd, argv);
		_exit(127);
	}
	return FALSE;
}

static void app_exit(void) {
	int status;
	gchar *pidfile = NULL;
	char* env_file = NULL;
	printf("t1");
	env_file = getenv(PID_ENV);
	if (env_file) {
		pidfile = (gchar*)g_strdup(getenv(PID_ENV));
	} else {
		pidfile = (gchar*)choices_find_path_save(PID_FILE, (const char*)app_name, FALSE);
	}
	printf("t2");
	status = unlink((const char*)pidfile);
	g_free(pidfile);
}

int main(int argc, char *argv[], char * envp[])
{
	GtkTooltips *tip = NULL;
	struct sigaction child_act, user_act;
	guint32 xid = 0;
	gint match;
	gboolean no_window;
	gchar *opt;
	char* env_file = NULL;

	pid_t pid;
	gchar *pidfile;
	FILE *pidstream;
#ifdef HAVE_BINDTEXTDOMAIN
	gchar *localedir;
#endif

#ifdef HAVE_SETLOCALE
	setlocale(LC_TIME, "");
	setlocale (LC_ALL, "");
#endif	
#ifdef HAVE_BINDTEXTDOMAIN
	/* More (untested) i18n support */
	localedir=g_build_path("/", g_getenv("APP_DIR"), "Messages", NULL);
	bindtextdomain(PROJECT, localedir);
	textdomain(PROJECT);
	g_free(localedir);
#endif
	/* Initialize global data with default values. */
	no_window=FALSE;
	app_name = getenv("APP_NAME");
	if (app_name == NULL) app_name = (guchar*) g_strdup(("ROX-Menu"));

	gtk_init(&argc, &argv);
	gdk_rgb_init();
	gtk_widget_push_visual(gdk_rgb_get_visual());
	gtk_widget_push_colormap(gdk_rgb_get_cmap());
	
	if (argc > 1) xid = (guint32) atol(argv[1]);
	if (xid) {
		window = gtk_plug_new(xid);
		if (!window) {
			rox_error(_("Failed to plug into socket %d! Not an XID?\n"), atol(argv[1]));
			app_exit();
			exit(1);
		} 
	} else {
		if (argc > 1) {
			opt = g_strdup("--no-window");
			match = g_ascii_strcasecmp(argv[1], opt);
			g_free(opt);
			no_window = match == 0 ? TRUE : FALSE;
		}
		if (no_window) window = NULL;
		else window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	}
	
	if (window) {
		gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
		gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy), NULL);
	}

	/* Let child processes die */
	child_act.sa_handler = child_died;
	sigemptyset(&child_act.sa_mask);
	child_act.sa_flags = SA_NOCLDSTOP;
	sigaction(SIGCHLD, &child_act, NULL);

	user_act.sa_handler = menu_popup;
	sigemptyset(&user_act.sa_mask);
	user_act.sa_flags = 0;
	sigaction(SIGUSR1, &user_act, NULL);
	sigaction(SIGUSR2, &user_act, NULL);
	sigaction(SIGTERM, &user_act, NULL);

	/* Set up the choices and options. */
	choices_init();
	default_choices_setup();  /* Make chocies dir, etc. */
	options_init(app_name);
	setup_options();
	options_notify();

	rox_dnd_init()  ;/* Initialize ROX drag-and-drop support. */

	/* Write the PID file. */
	printf("test1");
	env_file = getenv(PID_ENV);
	if (env_file) {
		pidfile = (gchar*)g_strdup(getenv(PID_ENV));
	} else {
		pidfile = (gchar*)choices_find_path_save(PID_FILE, (const char*) app_name, FALSE);
	}
	printf("test2");
	if (!g_file_test(pidfile, G_FILE_TEST_EXISTS)) {
		pid = getpid();
		pidstream = fopen((const char*)pidfile, "w");
		fprintf(pidstream, "%d", pid);
		fclose(pidstream);
		g_atexit((GVoidFunc)app_exit);
	}

	if (window) {
		gtk_window_set_title(GTK_WINDOW(window), app_name);
		button = gtk_button_new();
		gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
		GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(button), GTK_CAN_FOCUS);
		gtk_container_set_border_width(GTK_CONTAINER(button), 0);

		rox_dnd_register_uris(GTK_WIDGET(button), 0, 
	   	                   (rox_dnd_handle_uris)dnd_callback, NULL);
		 
		tip = gtk_tooltips_new();
		gtk_tooltips_set_tip(GTK_TOOLTIPS(tip), button,
		                     ("Click to start ROX applications."),
									("Displays a menu of applications.  Right-click to configure, refresh, or quit the menu."));

		change_menu_pixmap(LEAVE);

		gtk_widget_show(button);
		gtk_container_add(GTK_CONTAINER(window), button);

		gtk_widget_add_events(GTK_WIDGET(button), GDK_BUTTON_PRESS_MASK);
		gtk_signal_connect(GTK_OBJECT(button), "button_press_event", 
	   	                GTK_SIGNAL_FUNC(button_press), window);
		gtk_signal_connect_object(GTK_OBJECT(button), "enter", 
		     	                    GTK_SIGNAL_FUNC(change_menu_pixmap),
		                          GINT_TO_POINTER(ENTER));
		gtk_signal_connect_object(GTK_OBJECT(button), "leave", 
		                          GTK_SIGNAL_FUNC(change_menu_pixmap), 
		                          GINT_TO_POINTER(LEAVE));
	}
	
	build_menu();
	if (window) gtk_widget_show(window);

	gtk_main();

	return 0;
}
